package jrummikub.control.network;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import jrummikub.model.GameSettings;
import jrummikub.model.PlayerSettings;
import jrummikub.model.PlayerSettings.Type;
import jrummikub.util.Connection;
import jrummikub.util.GameData;
import jrummikub.util.IListener1;
import jrummikub.view.ISettingsPanel;
import jrummikub.view.ISettingsPanel.SettingsMode;
import jrummikub.view.IView;

public class GameJoinControl {
	private List<Connection> connections = new ArrayList<Connection>();
	private GameData gameData;
	private ConnectionControl connectionControl;
	private IView view;

	public GameJoinControl(final ConnectionControl connectionControl,
			final GameData gameData, final IView view) {
		this.connectionControl = connectionControl;
		this.gameData = gameData;
		this.view = view;

		view.getSettingsPanel().setSettingsMode(SettingsMode.NETWORK_JOIN);
		view.getSettingsPanel().enableAddPlayerButton(false);
		view.getSettingsPanel().setGameSettings(gameData.getGameSettings());

		connections.add(connectionControl.getGameOfferEvent().add(
				new IListener1<GameData>() {
					@Override
					public void handle(GameData data) {
						if (data.getGameID().equals(gameData.getGameID())) {
							GameSettings settings = data.getGameSettings();

							fixGameSettings(settings);

							gameData.setGameSettings(settings);

							updateSettingsPanel(settings);
						}
					}
				}));
	}

	private void updateSettingsPanel(GameSettings settings) {
		view.getSettingsPanel().setGameSettings(settings);

		Set<Color> colors = new HashSet<Color>(
				Arrays.asList(ISettingsPanel.PLAYER_COLORS));

		for (PlayerSettings player : settings.getPlayerList()) {
			colors.remove(player.getColor());
		}

		view.getSettingsPanel().setPlayerColors(colors);
	}

	private void fixGameSettings(GameSettings settings) {
		for (PlayerSettings player : settings.getPlayerList()) {
			if (player.getType() == Type.HUMAN) {
				player.setType(Type.NETWORK);
			} else if (player.getType() == Type.NETWORK
					&& player.getName().equals(connectionControl.getNickname())) {
				player.setType(Type.HUMAN);
			}
		}
	}

	public void startGameJoin() {
		view.showSettingsPanel(true);
	}

	public void abort() {
		// TODO Implement abort

		for (Connection c : connections) {
			c.remove();
		}
	}
}
